libname library 'c:';
data library.samuel;
   input id $ x y;
cards;
samuel 53024 42085
samuel 54050 42951
samuel 51280 42732
samuel 53191 43381
samuel 54951 43293
samuel 55073 43780
samuel 54658 42976
samuel 55854 42695
samuel 51500 43098
samuel 56378 43024
samuel 55659 42768
samuel 54744 42293
samuel 52524 42512
samuel 54524 42292
samuel 52073 42951
samuel 51085 41109
samuel 52439 43073
samuel 58938 47000
samuel 57436 48560
samuel 52595 42119
samuel 54667 43905
samuel 54761 43357
samuel 54952 43762
samuel 54952 43762
samuel 54881 43821
;
*------------------------------------------------------------------*
| SAS procedure to estimate and plot 95% Jennrich-Turner ellipse   |
| home range estimate, and perform a chi-square goodness of fit    |
| test.  The procedure uses PROC CORR to calculate the data sum-   |
| mary statistics needed, and uses a data step to calculate the    |
| home range and 95% confidence interval on this estimate.         |
| Then a second data step is used to calculate the points of       |
| the ellipse for plotting.  This data set is then combined and    |
| sorted with the original data to produce plots of the ellipse    |
| on the original data.  Finally, a goodness of fit test is        |
| performed in a separate data step.                               |
|                                                                  |
| A SAS data set is processed -- this data set should have the     |
| following variables defined:                                     |
|  id - Animal id used for BY statement processing                 |
|  x  - x coordinate                                               |
|  y  - y coordinate                                               |
|                                                                  |
|    Developed by: Gary C. White                                   |
|                  Department of Fishery and Wildlife Biology      |
|                  Colorado State University                       |
|                  Fort Collins, CO 80523                          |
|                  (303)491-6678                                   |
|                                                                  |
*------------------------------------------------------------------*;
libname library 'c:';
data datafile;
   set library.samuel;  /* <-- IMPORTANT -- specify SAS data file to
                                          process */
title 'Jennrich-Turner Home Range Estimation';
proc sort; by id date time;
proc corr cov noprint out=hrmss data=datafile; var x y; by id;
*------------------------------------------------------------------*
|   Data step to calculate the home range estimate                 |
*------------------------------------------------------------------*;
data jt; /* Data set JT contains the sufficient statistics to
            calculate the home range, and the estimate and CI. */
   set hrmss; by id;
   drop _type_ _name_ x y;
   retain xmean ymean xvar yvar xycov corr area lci uci n;
   if _type_ = 'COV' & _name_ = 'X' then do;
      xycov=y; xvar=x; end;
   if _type_ = 'COV' & _name_ = 'Y' then yvar=y;
   if _type_ = 'CORR' & _name_ = 'X' then corr=y;
   if _type_ = 'MEAN' then do;
      xmean=x; ymean=y; end;
   if _type_ = 'N' then n=min(x,y);
   if last.id then do;
      yvar=yvar*(n-1)/(n-2);
      xvar=xvar*(n-1)/(n-2);
      xycov=xycov*(n-1)/(n-2);
      area=(4*atan(1))*sqrt(xvar*yvar-xycov**2)*cinv(0.95,2);
      lci=(2*n-4)*area/cinv(0.975,2*n-4);
      uci=(2*n-4)*area/cinv(0.025,2*n-4);
      output;
      end;
proc print;
*------------------------------------------------------------*
|  Code to calculate a chi-square goodness of fit of the     |
|  bivariate normal distribution based on the number of      |
|  locations between similar concentric ellipses.            |
*------------------------------------------------------------*;
data gof;
   retain ncells 10;
   array observ{10} observ1-observ10;
   retain observ1-observ10 xmean xsd ymean ysd cost sint;
   keep id n chisq gofprob observ1-observ10;
   label chisq='Chi-square gof statistic'
         gofprob='Chi-square gof probability';
   merge jt datafile; by id;
   if first.id then do;
      ncells=min(int(n/5),10);
      do i=1 to ncells;
         observ{i}=0;
         end;
      do i=ncells+1 to 10;
         observ{i}=.;
         end;
   end;
   chi2=(((x-xmean)**2)*yvar-2*(x-xmean)*(y-ymean)*xycov+
         ((y-ymean)**2)*xvar)/(xvar*yvar-(xycov**2));
   d=sqrt(chi2);
   put x= y= chi2= d=;
   i=1;
   do while ( chi2 > cinv(i/ncells,2));
      i=i+1; if i = ncells then goto endloop;
      end;
endloop: observ{i}=observ{i}+1;
   if last.id then do;
      expect=n/ncells;
      chisq=0;
      do i=1 to ncells;
         chisq=chisq+(((observ{i}-expect)**2)/expect);
         end;
      gofprob=1-probchi(chisq,ncells-1);
      output;
   end;
proc print;
data combine;
   merge jt gof;by id;
   drop observ1-observ10;
proc print;
proc plot; plot area*chisq;
*---------------------------------------------*
|  Code to plot ellipse with original data    |
*---------------------------------------------*;
data ellipse;
   set jt;
   keep id x y;
   pi=4*atan(1);
   R=sqrt((xvar+yvar)**2-4*(xvar*yvar-xycov**2));
   theta=atan(-2*xycov/(yvar-xvar-R));
   a=sqrt((yvar+xvar+R)*cinv(0.95,2)/2);
   b=sqrt((yvar+xvar-R)*cinv(0.95,2)/2);
   cost=cos(theta);
   sint=sin(theta);
   do psi=0 to 360 by 2;
      psir=psi*pi/180;
      acosp=a*cos(psir);
      bcosp=b*sin(psir);
      x=xmean+acosp*cost-bcosp*sint;
      y=ymean+acosp*sint+bcosp*cost;
      output;
      end;
data plots;
   set datafile(in=indata) ellipse;
   if indata=1 then plotsym='+';
   else plotsym='*';
proc sort; by id;
proc plot; plot y*x=plotsym; by id;
run;
