*----------------------------------------------------------------*
| SAS procedure to determine minimum area polygon from set of    |
| biotelemetry data, calculate area, and plot polygon with       |
| original data.                                                 |
|                                                                |
| A SAS data set is processed -- this data set should have the   |
| following variables defined:                                   |
|  id - Animal id used for BY statement processing               |
|  x  - x coordinate                                             |
|  y  - y coordinate                                             |
|                                                                |
| Definitions of variables used in this program:                 |
|                                                                |
|  xcoor  - 1 dimension array of x coordinates                   |
|  ycoor  - 1 dimension array of y coordinates                   |
|  n      - number of coordinates in xcoor and ycoor             |
|  npoly  - number of points in polygon, sorted in clockwise     |
|           order to top of coor on return (0 means error)       |
|  area   - area in polygon                                      |
|  wsq     - Value of Cramer-von Mises goodness of fit statistic |
|  gofprob - Interpolated probability level of wsq               |
|                                                                |
|   Developed by: Gary C. White                                  |
|                 Department of Fishery and Wildlife Biology     |
|                 Colorado State University                      |
|                 Fort Collins, CO 80523                         |
|                 (303)491-6678                                  |
*----------------------------------------------------------------*;
title 'Minimum Area Polygon Calculation';
libname library 'c:';
data datafile;
   set library.bookex;  /* IMPORTANT -- Specify SAS data file here */
   keep id x y;
proc sort; by id;
data polygon;
   array xcoor{300} x1-x300;
   array ycoor{300} y1-y300;
   array z{300};
   array wcrit{5} w01 w025 w05 w10 w15;
   retain x1-x300 y1-y300 n;     /*          IMPORTANT           */
   set datafile; by id;     /* Specify data to process here */
   label area='Minimum convex polygon area in units**2'
         n='Number of telemetry locations'
         npoly='Number of corners in convex polygon'
         wsq='W**2 gof test'
         gofprob='Probability level W**2';
   keep id x y area n npoly wsq gofprob;
   if first.id then n=0;
   n=n+1;
   if n > 300 then do;
      put 'ERROR -- More than 300 observations for' id=;
      abort return 300;
      end;
   xcoor{n}=x; ycoor{n}=y;
   if last.id then do;
       /* find minimum y value, with maximum x value */
       /* i.e., the lower right corner coordinate    */
      imin=1;
      do i=2 to n;
         if ycoor{i} < ycoor{imin} then imin=i;
         else if ycoor{i} = ycoor{imin} & xcoor{i} > xcoor{imin} then imin=i;
      end;
      /*  put minimum y coordinate to position n+1 */
      xcoor{n+1}=xcoor{imin}; ycoor{n+1}=ycoor{imin};
      M=0; minangle=0;
      /* Start loop to find the rest of the corners */
      do until (imin=n+1);
        M=M+1;
        temp=xcoor{M}; xcoor{M}=xcoor{imin}; xcoor{imin}=temp;
        temp=ycoor{M}; ycoor{M}=ycoor{imin}; ycoor{imin}=temp;
        imin=n+1; v=minangle; minangle=360;
        do i=M+1 to n+1;
           link thetacal;
           if theta > v then do;
              if theta < minangle then do;
                 imin=i; minangle=theta; end;
              else if theta=minangle then do;
                 if ((xcoor{i}-xcoor{M})**2+(ycoor{i}-ycoor{M})**2) >=
                 ((xcoor{imin}-xcoor{M})**2+(ycoor{imin}-ycoor{M})**2)
                 then imin=i; end;
           end;
        end;
      end;
      npoly=M;
      area=0;
      if npoly >= 3 then do;
         area=xcoor{1}*(ycoor{npoly}-ycoor{2})+
              xcoor{npoly}*(ycoor{npoly-1}-ycoor{1});
         do i=2 to npoly-1;
            area=area+xcoor{i}*(ycoor{i-1}-ycoor{i+1});
            end;
         area=area*(-0.5);
         end;
      else area=.;
*----------------------------------------------------------------*
| SAS procedure to test for bivariate uniform data.              |
|                                                                |
|   Reference: Samuel, M. D. and E. O. Garton.  1985.  Home      |
|      range: a weighted normal estimate and tests of under-     |
|      lying assumtpions.  J. Wildl. Manage. 49(2):513-519.      |
|                                                                |
*----------------------------------------------------------------*;
      * Determine the rectangle that fits around the polygon;
      xu=xcoor{1}; xl=xcoor{1}; yu=ycoor{1}; yl=ycoor{1};
      do i=2 to npoly;
         xu=max(xu,xcoor{i});
         xl=min(xl,xcoor{i});
         yu=max(yu,ycoor{i});
         yl=min(yl,ycoor{i});
         end;
      seed=7654321; pi=4*atan(1);
      dbar=0;
      do i=1 to n;
         * Generate a random coordinate that is inside the polygon;
newval:  x=ranuni(seed)*(xu-xl)+xl;
         y=ranuni(seed)*(yu-yl)+yl;
         link inside;
         if not in then goto newval;
         distmin=sqrt((xcoor{1}-x)**2+(ycoor{1}-y)**2);
         do j=2 to n;
            distmin=min(distmin,sqrt((xcoor{j}-x)**2+(ycoor{j}-y)**2));
            end;
         * Calculate z{i} values for Cramer-von Mises test;
         z{i}=pi*distmin*distmin;
         dbar=dbar+z{i};
         end;
      dbar=dbar/n;
      do i=1 to n;
         z{i}=1-exp(-z{i}/dbar);
         end;
      link zsort;  /* Sort z array into ascending order */
      * Calculate critical values of this test.;
      den=1.0+0.6/n;
      w01=0.337/den;
      w025=0.273/den;
      w05=0.224/den;
      w10=0.177/den;
      w15=0.149/den;
      put @15 '****************' id= '****************';
      put @10 'Critical values of W**2 statistic';
      put @14 '(Listed in output as WSQ)';
      put @10 '---------------------------------';
      put @15 'W(0.01)  = ' w01;
      put @15 'W(0.025) = ' w025;
      put @15 'W(0.05)  = ' w05;
      put @15 'W(0.10)  = ' w10;
      put @15 'W(0.15)  = ' w15;
      link w2calc; wsq=wvalue; gofprob=prob;
      * Output polygon vertices to file;
      do i=1 to npoly;
         x=xcoor{i};
         y=ycoor{i};
         output;
         end;
      end;  /* end of do for last.id if */
   return;
*--------------------------------------------------------------*
|     This subroutine determines whether a point lies          |
|     inside or outside of a polygon (regular or irregular,    |
|     including concave and re-entrant).                       |
|     Written by Melvin L. Prueitt.  Version Aug. 25,1980.     |
|  npoly -  Number of vertices of the polygon.                 |
|  x     -  x coordinate of the point in question.             |
|  y     -  y coordinate of the point in question.             |
|  xcoor, ycoor are arrays giving the coordinates of the       |
|           polygon vertices.                                  |
|  in    - returned to the caller as a 1 if the point (x,y)    |
|          is inside the polygon, 0 if outside.  points on the |
|          boundary are considered to be outisde, but rounding |
|          errors sometimes cause boundary points to be inside.|
*--------------------------------------------------------------*;
inside: in=0;
      iii = npoly;
 s100:f = ycoor{1}-ycoor{iii};
      if f = 0 then do;
         iii = iii-1;
         if iii >= 1 then goto s100;
         else goto s600;
         end;
      else
        do iii=1 to npoly;
           jjj = mod(iii,npoly)+1;
           if (y-ycoor{iii})*(y-ycoor{jjj}) > 0 then goto s500;
           if y = ycoor{jjj} then goto s500;
           if y ^= ycoor{iii} then goto s400;
           if f*(ycoor{jjj}-y) > 0 then goto s500;
s400:      if x > (y-ycoor{iii})*(xcoor{jjj}-xcoor{iii})/
              (ycoor{jjj}-ycoor{iii})+xcoor{iii} then in = 1 - in;
s500:      if ycoor{jjj} ^= ycoor{iii} then f = ycoor{jjj}-ycoor{iii};
           end;
s600: return;
* Sort z{i} vector;
zsort:do i=1 to n-1;
         do j=i to n;
            if z{i} > z{j} then do;
               wvalue=z{i}; z{i}=z{j}; z{j}=wvalue; end;
            end;
         end;
    return;
* Calculate W**2 statistic;
w2calc: wvalue=0;
      do i=1 to n;
         wvalue=wvalue+(z{i}-(2*i-1)/(2*n))**2+(1/(12*n));
         end;
      link interp;
      return;
* Interpolate probability from critical values;
interp:if wvalue > wcrit{1} then prob=0.01;
      else if wvalue > wcrit{2} then
        prob=0.01+((wvalue-wcrit{1})/(wcrit{2}-wcrit{1}))*(0.025-0.01);
      else if wvalue > wcrit{3} then
        prob=0.025+((wvalue-wcrit{2})/(wcrit{3}-wcrit{2}))*(0.05-0.025);
      else if wvalue > wcrit{4} then
        prob=0.05+((wvalue-wcrit{3})/(wcrit{4}-wcrit{3}))*(0.10-0.05);
      else if wvalue > wcrit{5} then
        prob=0.10+((wvalue-wcrit{4})/(wcrit{5}-wcrit{4}))*(0.15-0.10);
      else prob=0.15;
    return;
* Determine theta -- a number between 0 and 360 that is not the ;
* angle made by point at M and point at i with the horizontal   ;
* but which has the same order properties as the true angle     ;
thetacal: dx=xcoor{i}-xcoor{M}; ax=abs(dx);
          dy=ycoor{i}-ycoor{M}; ay=abs(dy);
          if dx=0 & dy=0 then theta=0;
             else theta=dy/(ax+ay);
          if dx<0 then theta=2-theta;
             else if dy<0 then theta=4+theta;
          theta=theta*90;
     return;
proc print data=polygon;
/* Extra code to demonstrate some of the problems of sample size
   with Piceance deer data
data areas;
   set polygon; by id;
   keep id area n npoly;
   if first.id then output;
proc means; var area n;
proc plot; plot area*n;
proc chart; vbar area;
proc chart; vbar n;
proc chart; vbar npoly;                          */
*-------------------------------------------------------*
|   Code to plot polygon and merge with original data   |
*-------------------------------------------------------*;
data plots;
   retain firstx firsty lastx lasty;
   retain interval 25; /* Number of plot intervals along a side */
   keep x y id;
   set polygon; by id;
   if first.id then do;
      firstx=x; firsty=y;
      lastx=x;  lasty=y;
      end;
   else do;
      xnow=x; ynow=y;
      link line;
      lastx=xnow; lasty=ynow;
      end;
   if last.id then do;
      xnow=firstx; ynow=firsty;
      link line;
      end;
   return;
line: xinc=(xnow-lastx)/interval;
      yinc=(ynow-lasty)/interval;
      do i=1 to interval;
         x=lastx+xinc*i;
         y=lasty+yinc*i;
         output;
         end;
      return;
data both;
   set datafile (in=indata) plots; by id;
   if indata=1 then plotsym='+';
   else plotsym='*';
proc sort; by id;
proc plot data=both;
   plot y*x=plotsym;
   by id;
run;
