*----------------------------------------------------------------*
| SAS procedure to test for bivariate uniform data.              |
|                                                                |
|   Reference: Samuel, M. D. and E. O. Garton.  1985.  Home      |
|      range: a weighted normal estimate and tests of under-     |
|      lying assumtpions.  J. Wildl. Manage. 49(2):513-519.      |
|                                                                |
| A SAS data set is processed -- this data set must have the     |
| following variables defined:                                   |
|  id - Animal id used for BY statement processing               |
|  x  - x coordinate                                             |
|  y  - y coordinate                                             |
|                                                                |
| Definitions of variables used in this program:                 |
|                                                                |
|  xcoor   - 1 dimension array of x coordinates                  |
|  ycoor   - 1 dimension array of y coordinates                  |
|  n       - number of coordinates in xcoor and ycoor            |
|  wsq     - Value of Cramer-von Mises goodness of fit statistic |
|  gofprob - Interpolated probability level of wsq               |
|                                                                |
|   Developed by: Gary C. White                                  |
|                 Department of Fishery and Wildlife Biology     |
|                 Colorado State University                      |
|                 Fort Collins, CO 80523                         |
|                 (303)491-6678                                  |
*----------------------------------------------------------------*;
title 'Bivariate Uniform Test';
libname library 'c:';
data datafile;
   set library.samuel;  /* IMPORTANT -- specify SAS data file */
data wellipse;
   array xcoor{350} x1-x350;
   array ycoor{350} y1-y350;
   array z{350};
   array wcrit{5} w01 w025 w05 w10 w15;
   drop x1-x350 y1-y350 i j x y wvalue prob dbar xmin xmax
      ymin ymax xl xu yl yu pi seed w01 w025 w05 w10 w15 den
      xr yr dist distm;
   retain x1-x350 y1-y350 n;
   label wsq='W**2 gof test'
         gofprob='Probability level W**2';
   set datafile; by id;
   if first.id then n=0;
   n=n+1;
   if n > 350 then do;
      put 'ERROR -- More than 350 observations for' id=;
      abort return 350;
      end;
   xcoor{n}=x; ycoor{n}=y;
   if last.id then do;
      * Find min and max x and y locations;
      xmin=min(of x1-x350);
      ymin=min(of y1-y350);
      xmax=max(of x1-x350);
      ymax=max(of y1-y350);
 *   THESE BOUNDARIES SHOULD BE A MINIMUM CONVEX POLYGON;
      xu=xmin+0.90*(xmax-xmin);
      xl=xmin+0.10*(xmax-xmin);
      yu=ymin+0.90*(ymax-ymin);
      yl=ymin+0.10*(ymax-ymin);
      seed=7654321; pi=4*atan(1);
      dbar=0;
      do i=1 to n;
         xr=ranuni(seed)*(xu-xl)+xl;
         yr=ranuni(seed)*(yu-yl)+yl;
         distm=sqrt((xmax-xmin)**2+(ymax-ymin)**2);
         do j=1 to n;
            dist=sqrt((xcoor{j}-xr)**2+(ycoor{j}-yr)**2);
            if dist < distm then distm=dist;
            end;
         * Calculate z{i} values for Cramer-von Mises test;
         z{i}=pi*distm*distm;
         dbar=dbar+z{i};
         end;
      dbar=dbar/n;
      link zsort;  /* Sort z array into ascending order */
      do i=1 to n;
         z{i}=1-exp(-z{i}/dbar);
         end;
      * Calculate critical values of this test.;
      den=1.0+0.6/n;
      w01=0.337/den;
      w025=0.273/den;
      w05=0.224/den;
      w10=0.177/den;
      w15=0.149/den;
      put @15 '****************' id= '****************';
      put @10 'Critical values of W**2 statistic';
      put @14 '(Listed in output as WSQ)';
      put @10 '---------------------------------';
      put @15 'W(0.01)  = ' w01;
      put @15 'W(0.025) = ' w025;
      put @15 'W(0.05)  = ' w05;
      put @15 'W(0.10)  = ' w10;
      put @15 'W(0.15)  = ' w15;
      link w2calc; wsq=wvalue; gofprob=prob;
      output;
   end;
   return;
* Sort z{i} vector;
zsort:do i=1 to n-1;
         do j=i to n;
            if z{i} > z{j} then do;
               wvalue=z{i}; z{i}=z{j}; z{j}=wvalue; end;
            end;
         end;
    return;
* Calculate W**2 statistic;
w2calc: wvalue=0;
      do i=1 to n;
         wvalue=wvalue+(z{i}-(2*i-1)/(2*n))**2+(1/(12*n));
         end;
      link interp;
      return;
* Interpolate probability from critical values;
interp:if wvalue > wcrit{1} then prob=0.01;
      else if wvalue > wcrit{2} then
        prob=0.01+((wvalue-wcrit{1})/(wcrit{2}-wcrit{1}))*(0.025-0.01);
      else if wvalue > wcrit{3} then
        prob=0.025+((wvalue-wcrit{2})/(wcrit{3}-wcrit{2}))*(0.05-0.025);
      else if wvalue > wcrit{4} then
        prob=0.05+((wvalue-wcrit{3})/(wcrit{4}-wcrit{3}))*(0.10-0.05);
      else if wvalue > wcrit{5} then
        prob=0.10+((wvalue-wcrit{4})/(wcrit{5}-wcrit{4}))*(0.15-0.10);
      else prob=0.15;
    return;
proc print;
run;
