*----------------------------------------------------------------*
| SAS procedure to determine weighted bivariate normal ellipse   |
| estimate of home range, goodness of fit tests, and plot the    |
| weighted and unweighted ellipses on top of the original data.  |
|                                                                |
|   Reference: Samuel, M. D. and E. O. Garton.  1985.  Home      |
|      range: a weighted normal estimate and tests of under-     |
|      lying assumtpions.  J. Wildl. Manage. 49(2):513-519.      |
|                                                                |
| A SAS data set is processed -- this data set must have the     |
| following variables defined:                                   |
|  id - Animal id used for BY statement processing               |
|  x  - x coordinate                                             |
|  y  - y coordinate                                             |
|                                                                |
| Definitions of variables used in this program:                 |
|                                                                |
|  xcoor   - 1 dimension array of x coordinates                  |
|  ycoor   - 1 dimension array of y coordinates                  |
|  n       - number of coordinates in xcoor and ycoor            |
|  xmean   - mean of xcoor                                       |
|  ymean   - mean of ycoor                                       |
|  xvar    - variance of xcoor                                   |
|  yvar    - variance of ycoor                                   |
|  xycov   - covariance of xcoor and ycoor                       |
|  area    - area in Jennrich-Turner 95% ellipse                 |
|  lci     - lower 95% CI on area                                |
|  uci     - upper 95% CI on area                                |
|  wsq     - Value of Cramer-von Mises goodness of fit statistic |
|  gofprob - Interpolated probability level of wsq               |
|  wxmean  - weighted mean of xcoor                              |
|  wymean  - weighted mean of ycoor                              |
|  wxvar   - weighted variance of xcoor                          |
|  wyvar   - weighted variance of ycoor                          |
|  wxycov  - weighted covariance of xcoor and ycoor              |
|  warea   - area of robust Jennrich-Turner 95% ellipse          |
|  wlci    - lower 95% CI on warea                               |
|  wuci    - upper 95% CI on warea                               |
|  wwsq    - Value of Cramer-von Mises goodness of fit statistic |
|            for weighted estimates                              |
|  wgofprob- Interpolated probability level of wsq               |
|                                                                |
|   Developed by: Gary C. White                                  |
|                 Department of Fishery and Wildlife Biology     |
|                 Colorado State University                      |
|                 Fort Collins, CO 80523                         |
|                 (303)491-6678                                  |
*----------------------------------------------------------------*;
title 'Weighted Bivariate Normal Home Range Estimation';
libname library 'c:';
data datafile;
   keep id x y;
   set library.bookex;  /* IMPORTANT -- specify SAS data file */
proc sort; by id;
data wellipse;
   array xcoor{300} x1-x300;
   array ycoor{300} y1-y300;
   array z{300};
   array ctable{5,5};
   array wcrit{5} w01 w025 w05 w10 w15;
   drop x1-x300 y1-y300 w wtot i j x y wvalue prob dbar n1 delta
      oxmean oymean oxvar oyvar oxycov d d2 oarea iter
      w01 w025 w05 w10 w15 den;
   retain x1-x300 y1-y300 n;
   label xmean='x mean JT ellipse'
         ymean='y mean JT ellipse'
         xvar='x variance JT ellipse'
         yvar='y variance JT ellipse'
         xycov='covariance JT ellipse'
         area='area JT ellipse'
         wsq='W**2 gof test'
         wxmean='x mean robust JT ellipse'
         wymean='y mean robust JT ellipse'
         wxvar='x variance robust JT ellipse'
         wyvar='y variance robust JT ellipse'
         wxycov='covariance robust JT ellipse'
         warea='area robust JT ellipse';
   set datafile; by id;
   if first.id then n=0;
   n=n+1;
   if n > 300 then do;
      put 'ERROR -- More than 300 observations for' id=;
      abort return 300;
      end;
   xcoor{n}=x; ycoor{n}=y;
   if last.id then do;
      xmean=0; ymean=0;
      do i=1 to n;
         xmean=xmean+xcoor{i};
         ymean=ymean+ycoor{i};
         end;
      xmean=xmean/n;
      ymean=ymean/n;
      xvar=0; yvar=0; xycov=0;
      do i=1 to n;
         xvar=xvar+(xcoor{i}-xmean)**2;
         yvar=yvar+(ycoor{i}-ymean)**2;
         xycov=xycov+(xcoor{i}-xmean)*(ycoor{i}-ymean);
         end;
      xvar=xvar/(n-1);
      yvar=yvar/(n-1);
      xycov=xycov/(n-1);
      area=(4*atan(1))*sqrt(max(0,xvar*yvar-xycov**2))*cinv(0.95,2);
      lci=(2*n-4)*area/cinv(0.975,2*n-4);
      uci=(2*n-4)*area/cinv(0.025,2*n-4);
      oxmean=xmean; oymean=ymean; oxvar=xvar; oyvar=yvar;
      oxycov=xycov;
      * Calculate z{i} values for Cramer-von Mises test;
      dbar=0;
      do i=1 to n;
         link dcalc;
         z{i}=d2;
         dbar=dbar+z{i};
         end;
      dbar=dbar/n;
      do i=1 to n;
*        z{i}=probchi(z{i},2);  /* Why not same as next?? */
         z{i}=1-exp(-z{i}/dbar);
         end;
      link zsort;  /* Sort z array into ascending order */
      * Calculate critical values of this test.;
      den=1.0+0.6/n;
      w01=0.337/den;
      w025=0.273/den;
      w05=0.224/den;
      w10=0.177/den;
      w15=0.149/den;
      put @15 '****************' id= '****************';
      put @10 'Critical values of W**2 statistic';
      put @14 '(Listed in output as WSQ)';
      put @10 '---------------------------------';
      put @15 'W(0.01)  = ' w01;
      put @15 'W(0.025) = ' w025;
      put @15 'W(0.05)  = ' w05;
      put @15 'W(0.10)  = ' w10;
      put @15 'W(0.15)  = ' w15;
      link w2calc; wsq=wvalue; gofprob=prob;
      oarea=area; warea=0.9*area; iter=0;
      do while (abs(warea/oarea-1) > 0.00001);
         wxmean=0; wymean=0; wtot=0; oarea=warea; iter=iter+1;
         do i=1 to n;
            link dcalc;
            * put iter= i= d= w= ;
            z{i}=w;
            wxmean=wxmean+w*xcoor{i};
            wymean=wymean+w*ycoor{i};
            wtot=wtot+w;
            end;
         wxmean=wxmean/wtot;
         wymean=wymean/wtot;
         wtot=0; wyvar=0; wxvar=0; wxycov=0;
         do i=1 to n;
            w=z{i};
            wxvar=wxvar+w*w*(xcoor{i}-wxmean)**2;
            wyvar=wyvar+w*w*(ycoor{i}-wymean)**2;
            wxycov=wxycov+w*w*(xcoor{i}-wxmean)*(ycoor{i}-wymean);
            wtot=wtot+w*w;
            end;
         wxvar=wxvar/wtot;
         wyvar=wyvar/wtot;
         wxycov=wxycov/wtot;
         warea=(4*atan(1))*sqrt(max(0,wxvar*wyvar-wxycov**2))*cinv(0.95,2);
         * put wxmean= wymean= wxvar= wyvar= wxycov= warea=;
         oxmean=wxmean; oymean=wymean; oxvar=wxvar; oyvar=wyvar;
         oxycov=wxycov;
         end;
      wlci=(2*n-4)*warea/cinv(0.975,2*n-4);
      wuci=(2*n-4)*warea/cinv(0.025,2*n-4);
      *Perform goodness of fit test on weighted d2 values;
      do i=1 to n;
         link dcalc;
         z{i}=probchi(d2,1.96);
         end;
      link zsort;
      * Construct interpolation table for probability level of wwsq;
      ctable{1,1}=0.3003; ctable{1,2}=0.3370; ctable{1,3}=0.3022;
      ctable{1,4}=0.3015; ctable{1,5}=0.3351;
      ctable{2,1}=0.2452; ctable{2,2}=0.2967; ctable{2,3}=0.2588;
      ctable{2,4}=0.2519; ctable{2,5}=0.2701;
      ctable{3,1}=0.2142; ctable{3,2}=0.2278; ctable{3,3}=0.2173;
      ctable{3,4}=0.2060; ctable{3,5}=0.2224;
      ctable{4,1}=0.1711; ctable{4,2}=0.1737; ctable{4,3}=0.1662;
      ctable{4,4}=0.1707; ctable{4,5}=0.1760;
      ctable{5,1}=0.1496; ctable{5,2}=0.1518; ctable{5,3}=0.1398;
      ctable{5,4}=0.1485; ctable{5,5}=0.1495;
      * Interpolate value from table;
      if n > 200 then do;
         n1=5; delta=0; end;
      else if n <= 30 then do;
         n1=1; delta=0; end;
      else if n<= 50 then do;
         n1=2; delta=(n-50)/(50-30); end;
      else if n<= 70 then do;
         n1=3; delta=(n-70)/(70-50); end;
      else if n<= 100 then do;
         n1=4; delta=(n-100)/(100-70); end;
      else if n<= 200 then do;
         n1=4; delta=(n-200)/(200-100); end;
      do i=1 to 5;
         if n1 > 1 then
            wcrit{i}=ctable{i,n1}+(delta*(ctable{i,n1}-ctable{i,n1-1}));
         else
            wcrit{i}=ctable{i,n1};
         end;
      put @10 'Critical values for robust W**2 statistic';
      put @15 '(Listed as WWSQ on output)';
      put @10 '-----------------------------------------';
      put @15 'W(0.01)  = ' w01;
      put @15 'W(0.025) = ' w025;
      put @15 'W(0.05)  = ' w05;
      put @15 'W(0.10)  = ' w10;
      put @15 'W(0.15)  = ' w15;
      link w2calc; wwsq=wvalue; wgofprob=prob;
      output;
   end;
   return;
* Calculate Malanobis d statistic (also d**2 and w);
dcalc: d2=(((xcoor{i}-oxmean)**2)*oyvar
         -2*(xcoor{i}-oxmean)*(ycoor{i}-oymean)*oxycov+
           ((ycoor{i}-oymean)**2)*oxvar)
           /(oxvar*oyvar-(oxycov**2));
       d=sqrt(max(0,d2));
       if d > 1.97 then w=1.97/d;
                   else w=1.;
    return;
* Sort z{i} vector;
zsort:do i=1 to n-1;
         do j=i to n;
            if z{i} > z{j} then do;
               wvalue=z{i}; z{i}=z{j}; z{j}=wvalue; end;
            end;
         end;
    return;
* Calculate W**2 statistic;
w2calc: wvalue=0;
      do i=1 to n;
         wvalue=wvalue+(z{i}-(2*i-1)/(2*n))**2+(1/(12*n));
         end;
      link interp;
      return;
* Interpolate probability from critical values;
interp:if wvalue > wcrit{1} then prob=0.01;
      else if wvalue > wcrit{2} then
        prob=0.01+((wvalue-wcrit{1})/(wcrit{2}-wcrit{1}))*(0.025-0.01);
      else if wvalue > wcrit{3} then
        prob=0.025+((wvalue-wcrit{2})/(wcrit{3}-wcrit{2}))*(0.05-0.025);
      else if wvalue > wcrit{4} then
        prob=0.05+((wvalue-wcrit{3})/(wcrit{4}-wcrit{3}))*(0.10-0.05);
      else if wvalue > wcrit{5} then
        prob=0.10+((wvalue-wcrit{4})/(wcrit{5}-wcrit{4}))*(0.15-0.10);
      else prob=0.15;
    return;
proc print;
*---------------------------------------------------*
|  Code to plot both ellipses with original data    |
*---------------------------------------------------*;
data ellipse;
   set wellipse;
   keep id x y plotsym;
   pi=4*atan(1);
   xxmean=xmean; yymean=ymean; xxvar=xvar; yyvar=yvar;
   xxyycov=xycov; plotsym='U';
   do i=1 to 2;
      R=sqrt(max(0,(xxvar+yyvar)**2-
         4*(xxvar*yyvar-xxyycov**2)));
      theta=atan(-2*xxyycov/(yyvar-xxvar-R));
      a=sqrt(max(0,(yyvar+xxvar+R)*cinv(0.95,2)/2));
      b=sqrt(max(0,(yyvar+xxvar-R)*cinv(0.95,2)/2));
      cost=cos(theta);
      sint=sin(theta);
      do psi=0 to 360 by 3;
         psir=psi*pi/180;
         acosp=a*cos(psir);
         bcosp=b*sin(psir);
         x=xxmean+acosp*cost-bcosp*sint;
         y=yymean+acosp*sint+bcosp*cost;
         output;
         end;
      xxmean=wxmean; yymean=wymean; xxvar=wxvar; yyvar=wyvar;
      xxyycov=wxycov; plotsym='W';
      end;
data plots;
   set datafile(in=indata) ellipse;
   if indata=1 then plotsym='+';
proc sort; by id;
proc plot; plot y*x=plotsym; by id;
run;
